/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef AVR_PORT_ENUM_H
#define AVR_PORT_ENUM_H

/***********************************************************/
 
#ifndef ID_TO_BIT
#define ID_TO_BIT(pinid) \
	(1<<((pinid)&7))
#endif /* ID_TO_BIT */	

#define PIN_ID(c,n) \
	((PORT ## c ## _ID << 3) | (n))
	
#define ENUM_PIN(letter,pin) \
	PORT ## letter ## _ ## pin = PIN_ID(letter,pin)

#define ENUM_PORT(p) \
	ENUM_PIN (p,0), \
	ENUM_PIN (p,1), \
	ENUM_PIN (p,2), \
	ENUM_PIN (p,3), \
	ENUM_PIN (p,4), \
	ENUM_PIN (p,5), \
	ENUM_PIN (p,6), \
	ENUM_PIN (p,7)
	
/*
 * Enumerate all ports of respective MCU,
 * giving us the desired values like PORTD_1 or PORTB_4, e.g.
 */

#if defined(__AVR_AT90S2313__)	\
	|| defined (__AVR_ATtiny2313__)
enum {
	ENUM_PORT (B),
	
	ENUM_PIN (D,0),
	ENUM_PIN (D,1),
	ENUM_PIN (D,2),
	ENUM_PIN (D,3),
	ENUM_PIN (D,4),
	ENUM_PIN (D,5),
	ENUM_PIN (D,6),
	
#if defined (__AVR_ATtiny2313__)
	ENUM_PIN (A,0),
	ENUM_PIN (A,1),
#endif 
	ENUM_PIN_LAST
};
#elif defined(__AVR_ATmega8__) \
      || defined(__AVR_ATmega88__) \
      || defined(__AVR_ATmega168__) \
      || defined(__AVR_ATmega328P__)
enum {
	ENUM_PORT (B),
	
	ENUM_PIN (C,0),
	ENUM_PIN (C,1),
	ENUM_PIN (C,2),
	ENUM_PIN (C,3),
	ENUM_PIN (C,4),
	ENUM_PIN (C,5),
	ENUM_PIN (C,6),

	ENUM_PORT (D),
	
	ENUM_PIN_LAST
};
#elif defined(__AVR_AT90S2333__) \
      || defined(__AVR_AT90S4433__)
enum {
	ENUM_PIN (B,0),
	ENUM_PIN (B,1),
	ENUM_PIN (B,2),
	ENUM_PIN (B,3),
	ENUM_PIN (B,4),
	ENUM_PIN (B,5),
	
	ENUM_PIN (C,0),
	ENUM_PIN (C,1),
	ENUM_PIN (C,2),
	ENUM_PIN (C,3),
	ENUM_PIN (C,4),
	ENUM_PIN (C,5),

	ENUM_PORT (D),
	
	ENUM_PIN_LAST
};
#elif defined(__AVR_AT90S4414__) \
      || defined(__AVR_AT90S8515__) \
      || defined(__AVR_AT90S4434__) \
      || defined(__AVR_AT90S8535__) \
      || defined(__AVR_ATmega163__)
enum 
{
	ENUM_PORT (A),
	ENUM_PORT (B),
	ENUM_PORT (C),
	ENUM_PORT (D),
	
	ENUM_PIN_LAST
};
#elif defined(__AVR_ATtiny25__) \
      || defined(__AVR_ATtiny45__) \
      || defined(__AVR_ATtiny85__)

enum 
{
	ENUM_PIN (B,0),
	ENUM_PIN (B,1),
	ENUM_PIN (B,2),
	ENUM_PIN (B,3),
	ENUM_PIN (B,4),
	
	ENUM_PIN_LAST
};	
#elif defined(__AVR_ATtiny24__) \
      || defined(__AVR_ATtiny44__) \
      || defined(__AVR_ATtiny84__)
enum
{		
	ENUM_PORT (A),
    
	ENUM_PIN (B,0),
	ENUM_PIN (B,1),
	ENUM_PIN (B,2),
	
	ENUM_PIN_LAST
};	

#else
#error Please define available ports for MCU
#endif

#endif /* AVR_PORT_ENUM_H */
