/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include "compat/interrupt.h"
#include "attrib.h"
#include "frame.h"

frame_t frame;

framebuf_t NOINIT framebuf;

void CONSTRUCTOR 
init_frame (void)
{
    FLAGS = 0;
    frame.schoner_timeout = FRAME_SCHONER_TIMEOUT;
}

// Zeiger auf die Beam-Puffer initialisieren
void 
frame_bufinit (frame_t *b, uint16_t size)
{
    uint8_t *buf = (uint8_t*) & framebuf;
    
    FLAGS &= ~FRAME_ISR_NEW;
    FLAGS &= ~FRAME_VALID;
    
    b->buffers.isr      = buf;
    b->buffers.tmp      = buf+size;
    b->buffers.tmp_new  = (buf+size) +size;
    b->buffers.isr_new  = (buf+size) +size;
}

// Beam-Daten sind fertig: aktivieren
void 
frame_activate (frame_t *b)
{
    uint8_t sreg = SREG;
    cli();
    
    void * tmp = (IS_FLAG (FRAME_ISR_NEW))
                    // der .isr_new-Datensatz wurde von der ISR nicht abgeholt
                    ? b->buffers.isr_new
                    // der .isr_new-Datensatz wurde von der ISR abgeholt
                    : b->buffers.tmp_new;
                    
    b->buffers.isr_new = b->buffers.tmp;
    b->buffers.tmp     = tmp;

    FLAGS |= FRAME_VALID;
    FLAGS |= FRAME_ISR_NEW;
    
    SREG = sreg;
}

#define SIZELINE(X) \
    asm volatile (";;sizeof " #X ": \t%0\n\t" :: "n" (sizeof (framebuf.X)))

void NAKED dummy (void)
{
    SIZELINE (uhr);
    SIZELINE (snake);
    SIZELINE (text);
    SIZELINE (test);
    SIZELINE (oszi);
    SIZELINE (asteroids);
}
