/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include "compat/interrupt.h"

#ifdef __AVR__

#   include <avr/io.h>
#   include <avr/wdt.h>

#   include "dac.h"
#   include "morse.h"
#   include "ports.h"
#   include "panic.h"
#   ifdef DEBUG
#      include "uart.h"
#      include "uput.h"
#   else // !DEBUG
#       define put_P(...) (void) 0
#       include "ports.h"
#   endif // DEBUG
#   include "sound.h"
#   include "rc5-pd.h"

static void pixel_on_the_fly (void);

#elif defined i386
#warning i386
//#   include "vecfont-pc-compat.h"

#endif 

#include "menu.h"
#include "countdown.h"
#include "attrib.h"
#include "reload.h"

#include "main.h"
#include "frame.h"

//#define put_P(...)

system_t sys;

/*****************************************************************/
/*****************************************************************/
#define FB_REG 1
void 
pixel_on_the_fly (void)
{
#if FB_REG && __GNUC__ < 4 && defined __AVR__
    // Binde fb an Y-Register. Dies geht nur fr gcc 3
    register frame_t *fb asm ("r28") = & frame;
#else
    //frame_t *fb = & frame;
    #define fb (&frame)
#endif
    
    // tempus fugit
    fb->ticks++;

    //////////////////////////////////////////////////
    
    int8_t off = fb->off;
    
    // TILT?
    if (!IS_FLAG (FRAME_VALID)
        || !count.sec.schoner
        || sys.bootlevel < BOOT_DONE)
    {
        // Evtl. Sperrung aufheben
        FLAGS &= ~FRAME_LOCKED;

        // Strahl an/austasten
        TOGGLE (PORT_Z);

        // und ins Aus stellen
        reg_pixel = (upixel8_t) {0,0};
        FLAGS |= BEAM_TILT;
    }
    else
    {
        // bergang von TILT -> On bewerkstelligen
        if (IS_FLAG (BEAM_TILT))
        {
            // FIXME: OFF-Sequenz simulieren
            SET (PORT_Z);
            FLAGS &= ~BEAM_TILT;
            FLAGS |= BEAM_GET_PIXEL;
            FLAGS |= BEAM_SKIP;
            off = 0;
        }
    }

    // Pixel-Ausgabe, -Berechnung, Strahl-Austastung und Off-Zeit

    if (off == 0
        || IS_FLAG (BEAM_GET_PIXEL)
        || IS_FLAG (BEAM_TILT))
    {
        FLAGS &= ~BEAM_GET_PIXEL;
        
        // Koordinaten in reg_pixel auf Rhre (bzw. an DAC) ausgeben
        dac_p();
        
        if (IS_FLAG (BEAM_TILT))
            return;
            
        // Koordinaten fr evtl. Abstandsbestimmung merken
        upixel8_t p = reg_pixel;

        // WARNUNG: BEAM_SKIP ist nur ein Alias fr BEAM_GET_PIXEL!
        FLAGS &= ~BEAM_SKIP;
        FLAGS &= ~BEAM_CHANGED;
        
        do
        {
            void * bisr = fb->buffers.isr;
            
            // Die Anwendung hat einen Datensatz fertig
            if (IS_FLAG (FRAME_ISR_NEW)
                && IS_FLAG (FRAME_VALID)
                && !IS_FLAG (FRAME_LOCKED))
            {
                // bernehmen, sofern der Strahl nicht geLOCKt ist
                fb->buffers.tmp_new  = bisr;
                fb->buffers.isr      = bisr = fb->buffers.isr_new;
                // Der Anwendung signalisieren, da der Frame abgeholt
                // wurde und .tmp_new frei geworden ist.
                FLAGS &= ~FRAME_ISR_NEW;
            }

            // Neue Koordinaten bestimmen
            if (IS_FLAG (FRAME_LOCKED)
                || fb->ticks < 10)
            {
                ((onPixel_t) menu.item.onPixel) (menu.item.data, bisr);
            }
            else
            {
                if (fb->ticks >= 1000)
                    fb->ticks = 0;
                    
                reg_pixel = (upixel8_t) {0,0};
                FLAGS |= BEAM_SKIP;
                break;
            }
            
        } while (!IS_FLAG (BEAM_CHANGED));
        
        int8_t off2 = 0;
        
        if (IS_FLAG (BEAM_SKIP))
        {   
            FLAGS &= ~BEAM_SKIP;
            // Distanzbestimmung
            // Off-Zeit aus | p - reg_pixel | berechnen
            // dx = | p.x - reg_pixel.x |
            uint8_t dx = p.x-reg_pixel.x;   if (p.x < reg_pixel.x)    dx = -dx;
            uint8_t dy = p.y-reg_pixel.y;   if (p.y < reg_pixel.y)    dy = -dy;
            
            off2 = -1;
            // Abstand durch 2 teilen, damit er in 16 Bits passt
            uint16_t dist16 = dx*(dx>>1) + dy*(dy>>1);
            
            if (dist16 >= 8*8)                { off2 -= 1;
                uint8_t  dist8 = dist16 >> 8;
                    
                if (dist8 >=  25*25  / 0x100) { off2 -= 1;
                if (dist8 >=  50*50  / 0x100) { off2 -= 1;
                if (dist8 >=  70*70  / 0x100) { off2 -= 2;
                if (dist8 >= 150*150 / 0x100) { off2 -= 3;
            }}}}}
        }
        
        if (off == 0)
            // off < 0 triggert im nchsten ISR Start der Off-Zeit
            off = off2;
        else
            // Noch innerhalb der Off-Zeit: Wert zurckhalten fr spter
            fb->off2 = off2;
    }
    else if (off < 0)
    {
        // Im vorherigen ISR-Frame wurde off < 0 gesetzt:
        // Nun beginnt der Countdown fr die Off-Zeit
        off = -off;
        SET (PORT_Z);
        // Fr den nchsten Frame die whrend der Off-Zeit zu setzenden
        // Koordinaten anfordern
        FLAGS |= BEAM_GET_PIXEL;
    }
    else if (off > 0)
    {
        // In der Off-Zeit
        if (--off == 0)
        {
            // Off-Zeit abgelaufen
            // Strahl wieder eintasten
            CLR (PORT_Z);
            
            if (fb->off2)
            {
                // Im Falle einer zurckgehaltenen Off-Zeit
                // diese nun bernehmen, um im nchsten ISR-Frame
                // die Austastung zu starten
                off = fb->off2;
                fb->off2 = 0;
            }
        }
    }
    
    fb->off = off;
}
#undef fb

#if defined __AVR__

static void 
job_dcf (void)
{
    uint8_t bit = 0;
    
    if (IS_SET (PORT_DCF))
        bit = 1;
        
    menu.dcf_bit = bit;
}

//EMPTY_INTERRUPT (SIG_WATCHDOG_TIMEOUT)
//{
//    panic (_W_);
//}

#define PRESCALE 8

static const uint8_t fcpu_err[] PROGMEM = 
{
#if (F_CPU % (PRESCALE * IRQS)) * 2 == PRESCALE * IRQS
    #define ERR_NUM    1
    #define ERR_CYCLES 2
#elif (F_CPU % (PRESCALE * IRQS)) * 3 == PRESCALE * IRQS
    #define ERR_NUM    1
    #define ERR_CYCLES 3
#elif (F_CPU % (PRESCALE * IRQS)) * 3 == 2 * PRESCALE * IRQS
    #define ERR_NUM    2
    #define ERR_CYCLES 3
#else
#if (F_CPU % (PRESCALE * IRQS)) != 0 
#   warning Timer2 arbeitet ungenau
#endif
    #define ERR_NUM    0
    #define ERR_CYCLES 0
#endif
};

//#undef ERR_CYCLES
//#define ERR_CYCLES 0

SIGNAL (SIG_OUTPUT_COMPARE2A)
{
    sys.stat.loop = 0;
    
    while (1)
    {
        wdt_reset();

        // Fehler aufgrund nicht-passener IRQ-Rate ausbgeln, d.h.
        // wenn F_CPU % (PRESCALE * IRQs) != 0 ist. Der verbleibenede
        // Rest wird in PWM-hnlicher Manier auf die IRQ-Rate aufgeschlagen.
        if (ERR_CYCLES)
        {
            static uint8_t isr_error;
            uint8_t ocr = (uint32_t) F_CPU / PRESCALE / IRQS-1;
            uint8_t err = 1 + isr_error;

            if (ERR_CYCLES == 2)
            {
                err %= 2;
                ocr += err;
            }
            else 
            {    
                if (err <= ERR_NUM)           ocr++;
                if (err >= ERR_CYCLES)        err = 0;
            }
            
            isr_error = err;
                
            OCR2A = ocr; 
        } // ERR_CYCLES
        
        pixel_on_the_fly ();
        rc5_poll();
        
#if FB_REG && __GNUC__ < 4 && defined __AVR__
        asm volatile (";":::"28","29");
#endif
        
        ////////////////////////////////////////////////////////////
        // 1 ms-Takt fr onRedraw
        uint8_t irqs_1ms = 1 + sys.irq.num_1ms;
        if (irqs_1ms >= IRQS / 1000)
        {
            // 1 ms ist voll
            menu.tick_1ms = 1;
            irqs_1ms = 0;
        }
        sys.irq.num_1ms = irqs_1ms;	
        
        ////////////////////////////////////////////////////////////
        // 10 ms-Takt fr onRedraw
        // 10 ms-Takt fr Jobs: countdown-Zhler, Tasten-Entprellung, DCF, ...
        uint16_t irqs_10ms = 1 + sys.irq.num_10ms;
        if (irqs_10ms >= IRQS / 100)
        {
            // 10 ms sind voll
            menu.tick_10ms = 1;
            irqs_10ms = 0;
        }
            
        sys.irq.num_10ms = irqs_10ms;	
        
        if (irqs_10ms <= 7)
        {
            uint8_t i = irqs_10ms;

            // Immer Lcken zum Erholen lassen
            if (i == 1)   job_countdown();
            if (i == 3)   job_morse (&morse_data);
            if (i == 5)   job_sound (&sdata);
            if (i == 7)   job_dcf ();
        }
        
        if (! (TIFR2 & (1 << OCF2A)))
            break;
            
        // Was? Schon so spt?
        if (++sys.stat.loop >= 10)
            break;
            
        TIFR2 = (1 << OCF2A);
        // Direkt zurckspringen spart einen ISR-Epilog und -Prolog,
        // die zusammen rund 90 Ticks kosten wrden!
    }
  
//    TIMSK2 = (1 << OCIE2A);
}

/*****************************************************************************************/
/*****************************************************************************************/
/*****************************************************************************************/

static void 
timer2_init (void)
{
    // Mode #2 (CTC) und PRESCALE=8
    TCCR2A = (1 << WGM21);
    TCCR2B = (1 << CS21);

    // OutputCompare fr gewnschte Timer2 Frequenz
    OCR2A = (unsigned short) ((unsigned long) F_CPU / PRESCALE / IRQS-1);

    // OutputCompare-Interrupt A fr Timer 2
    TIMSK2 = (1 << OCIE2A);
}

static void 
ioinit (void)
{
	// unbenutzte Ports
	SET (PORTD_7);
	SET (PORTB_0);
	SET (PORTB_3);
    
    // Hochspannung aus
    CLR (PORT_POWER);
    MAKE_OUT (PORT_POWER);
    
    SET (PORT_DREHTAST);
    SET (PORT_DREHA);
    SET (PORT_DREHB);
    
	MAKE_OUT (PORT_Z);
    
	timer2_init();

#ifdef UART_H	
	MAKE_OUT (PORT_TxD);
	SET (PORT_TxD);
	MAKE_IN  (PORT_RxD);
	SET      (PORT_RxD);
	uart_init();
#else  // UART_H	
	MAKE_IN (PORT_TxD);	SET (PORT_TxD);
	MAKE_IN (PORT_RxD);	SET (PORT_RxD);
	MAKE_IN (PORT_DCF);	CLR (PORT_RxD);
#endif // UART_H
}

/*****************************************************************************************/
/*****************************************************************************************/

void INIT(3) 
init3_main(void)
{
    sys.mcusr = MCUSR;
    MCUSR = 0;
}

void INIT(8) 
init8_main(void)
{
    sys.bootlevel = BOOT_INIT;
}

int 
main (void)
{
    sys.bootlevel = BOOT_SETUP;
    
    if (sys.mcusr & (1 << WDRF))
        panic (_W_);
        
    if (sys.mcusr & (1 << BORF))
        panic (_B_);
    
    ioinit (); 

    sei();
    
    wait_10ms (3);
    
    put_P ("\nHallo Morpheus!\n");
    
    menu_main (&menu);
}

#endif // AVR
