/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#ifndef MORSE_H
#define MORSE_H

#include <avr/io.h>

#define MORSE_DOT  5

static inline void 
speaker_on(void)
{
    // Die beiden PWM-Ports arbeiten gegengleich
    TCCR1A = (1 << COM1B1) | (1 << COM1A1) | (1 << COM1A0);
}

static inline void 
speaker_off(void)
{
    // PWM-Ports deaktivieren
    TCCR1A = 0;
}

typedef struct
{
    uint8_t buch;
    uint8_t volatile bits;
    uint8_t stage, wait;
} morse_t;

extern morse_t morse_data;

extern void job_morse (morse_t *);
extern void morse (uint8_t);

extern void morse_init (void);

// Definitionen fr die Buchstaben,
// um nicht die gesamte Tabelle im Speicher zu haben.
// Dies vergrssert den Quelltext, nicht das Programm.
// 0x00 und 0x20 tauchen in der Aufzhlung nicht auf.
// Die Codierung eines Zeichens befindet sich in den 
// Bits 0..6, ds erste Bit ist immer 1.
// Zeichen beginnt mit einem DIT:
//     Bit 7 = 1
//     DIT = 1
//     DAH = 0
// Zeichen beginnt mit einem DAH:
//     Bit 7 = 0
//     DIT = 0
//     DAH = 1
// Damit sind 2*128 = 254 Zeichen darstellbar.
// Ungltige Zeichen sind 0x00 und 0x80, weil dann
// im MSB von 0..6 kein Bit gesetzt ist, das den Anfang
// des Morse-Codes kennzeichnet.
#define _A_ 0x82 /* A = .- */
#define _B_ 0x08 /* B = -... */
#define _C_ 0x0a /* C = -.-. */
#define _D_ 0x04 /* D = -.. */
#define _E_ 0x81 /* E = . */
#define _F_ 0x8d /* F = ..-. */
#define _G_ 0x06 /* G = --. */
#define _H_ 0x8f /* H = .... */
#define _I_ 0x83 /* I = .. */
#define _J_ 0x88 /* J = .--- */
#define _K_ 0x05 /* K = -.- */
#define _L_ 0x8b /* L = .-.. */
#define _M_ 0x03 /* M = -- */
#define _N_ 0x02 /* N = -. */
#define _O_ 0x07 /* O = --- */
#define _P_ 0x89 /* P = .--. */
#define _Q_ 0x0d /* Q = --.- */
#define _R_ 0x85 /* R = .-. */
#define _S_ 0x87 /* S = ... */
#define _T_ 0x01 /* T = - */
#define _U_ 0x86 /* U = ..- */
#define _V_ 0x8e /* V = ...- */
#define _W_ 0x84 /* W = .-- */
#define _X_ 0x09 /* X = -..- */
#define _Y_ 0x0b /* Y = -.-- */
#define _Z_ 0x0c /* Z = --.. */

#define _0_ 0x1f /* 0 = ----- */
#define _1_ 0x90 /* 1 = .---- */
#define _2_ 0x98 /* 2 = ..--- */
#define _3_ 0x9c /* 3 = ...-- */
#define _4_ 0x9e /* 4 = ....- */
#define _5_ 0x9f /* 5 = ..... */
#define _6_ 0x10 /* 6 = -.... */
#define _7_ 0x18 /* 7 = --... */
#define _8_ 0x1c /* 8 = ---.. */
#define _9_ 0x1e /* 9 = ----. */

#define _FRAGE_ 0xb3 /* ? = ..--.. */
#define _AUSRUF_ 0x33 /* ! = --..-- */

#endif /* MORSE_H */
