#include "compat/pgmspace.h"

#include "snake.h"
#include "reload.h"
#include "frame.h"

// 1 Pixel aufs Rohr
void
onPixel_snake (snake_t * s UNUSED, snake_frame_t * sb)
{
    // Texte, Mauer, Schlange (Kopf, Body), Futter, etc. zeichnen
    // elem =
    //    TEXT <-- fa, fb
    //    WALL <-- fc, fd
    //    FOOD <-- fe
    //    HEAD <-- ff
    //    BODY <-- 0...nseg-1

    // # noch zu zeichnender Pixel fr dieses Element
    uint8_t len  = sb->draw.len;
    uint8_t x    = sb->draw.x;
    uint8_t y    = sb->draw.y;
    uint8_t elem = sb->draw.elem;
    
    FLAGS |= FRAME_LOCKED;

    if (len == 0)
    {
    _next_element:;
    
        // Elemenet fertig: nchstes Segment
        elem++;
        if ((int8_t) elem >= (int8_t) sb->body.nseg)
        {
            // Letztes Element ist gezeichnet: dieser Frame ist fertig
            sb->draw.elem = 0xfa-1;
            
            FLAGS &= ~FRAME_LOCKED;
            return;
        }
        
        sb->draw.elem = elem;
        
        // Springt der Strahl?
        uint8_t skip = 1;
        
        if (elem >= 0xfe)
        {
            // Koordinaten fr Kopf, Futter, ...
            uint8_t idx = ~elem;
            sb->draw.x = x = sb->p[idx].x;
            sb->draw.y = y = sb->p[idx].y;
        }

        if (elem < 0x80)
        {
            //  Krper-Segment
            uint8_t idx = elem + sb->body.start;

            if (idx >= SEG_MAX)
                idx -= SEG_MAX;

            // Richtung und Lnge
            sb->draw.dir = sb->body.seg[idx].dir;
            len = sb->body.seg[idx].len;
            skip = 0;
        }
        else if (elem == 0xff-SN_HEAD)
        {
            if (!sb->body.nseg)
                goto _next_element;
            // Kopf
            sb->draw.dir = sb->dir;
            sb->draw.len = 7;
            skip = 1;
        }
        else if (elem == 0xff-SN_FOOD)
        {
            if (!sb->food.x)
                goto _next_element;
            sb->draw.len = 9;
            skip = 1;
        }
        else if (elem == 0xfa)
        {
            // Text init
            uint8_t n = sb->draw.ntext;
            
            const uint8_t * tinit = (const uint8_t*) & snake_text_init[n];
            // .start
            vecfont.start.x = pgm_read_byte_inc (tinit);
            uint8_t y = pgm_read_byte_inc (tinit);
            if (n == 2)
                y = snake.scroll_y;
            vecfont.start.y = y;
            // .zoom
            vecfont.zoom          = pgm_read_byte_inc (tinit);
            // .text_in_flash
            vecfont.text_in_flash = pgm_read_byte_inc (tinit);
            // .text
            const char * pstr = sb->str;
            
            if (n)
                pstr = (const char*) pgm_read_addr (tinit);

            vecfont.text = pstr;
            // .proportional
            vecfont.proportional = 1;
            skip = 0;
        }
        else if (elem == 0xfb)
        {
            // Text
            sb->draw.elem = --elem;
            skip = 0;
        }
        else if (elem == 0xfc)
        {
            // Mauer-Anfang
            sb->draw.wall = snake_wall;
            snake_read_wall_seg (sb);
            
            skip = 1;
        }
        else if (elem == 0xfd)
        {
            // Mauer
            if (snake_read_wall_seg (sb))
            {
                if (sb->draw.elem == 0xfd)
                    return;
                    
                skip = 1;
            }
            else
            {
                // read_wall_seg vernderte x, y, len: aktualisieren
                x   = sb->draw.x;
                y   = sb->draw.y;
                len = sb->draw.len;
                elem = 0xfd-1;
                skip = 0;
            }
        }

        if (skip)
        {
            // Strahl austasten
            beam_skip();
            beam_xy (sb->draw.x, sb->draw.y);
            return;
        }
        
    } // len == 0
    
    ///////////////////////////////////////////////////////////////

    if (elem == 0xfb-1)
    {
        // Text
        uint8_t text_done = vecfont_draw_letters (& vecfont);
        sb->draw.len = !text_done;
        if (text_done)
        {
            if (++ sb->draw.ntext >= sb->ntexte)
            {
                sb->draw.ntext = 0;
                sb->draw.elem  = 0xfc-1;
            }
            else
                sb->draw.elem = 0xfa-1;
        }
        return;
    }

    uint8_t dir = sb->draw.dir;
    
    if (elem == 0xfd-1)
    {
        // Mauer zeichnen
        // (1 << SNAKE_WALL_SKIP) Schritte in Segment-Richtung fortschreiten
        uint8_t wskip = 1 << SNAKE_WALL_SKIP;
        sb->draw.len = len - wskip;
        if (dir < 2)    wskip = -wskip;
        if (dir & 1)    y += wskip;
        else            x += wskip;

        // Wiederhole; read_wall_seg() kmmert sich um Mauer-Ende
        sb->draw.elem = 0xfd-1;

        sb->draw.x = x;
        sb->draw.y = y;
        // Punkt auf die Rhre
        beam_xy (x, y);
        return;
    }

    sb->draw.len = --len;

    if (elem >= 0xfe)
    {
        // Pixel von Kopf, Futter, ... zeichnen

        uint16_t dxy = pgm_read_word ((uint16_t*) & snake_head_dxy[len]);

        uint8_t dx = dxy;
        uint8_t dy = dxy >> 8;

        if (dir < 2)            dx = -dx, dy = -dy;
        if (!(dir & 1))         dx ^= dy, dy ^= dx, dx ^= dy;

        x += dx;
        y += dy;
    }
    else if (elem < 0x80)
    {
        // Body zeichnen
        // 1 Schritt in Segment-Richtung fortschreiten
        if (dir & 1)    y += dir-2;
        else            x += dir-1;

        sb->draw.x = x;
        sb->draw.y = y;
    }

    // Punkt auf die Rhre
    beam_xy (x, y);
}
