/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "sound.h"

#define FREQ(X) (F_CPU/(2*(X)-1))

#define BASE 440

#define C0      1.0
#define D0      1.122
#define E0      1.26
#define FIS0    1.414
#define G0      1.498
#define A0      1.682
#define H0      1.888
#define C1      2*1.0
#define D1      2*1.122

#define SOUND_ON  do { TCCR1A = (1 << COM1B1) | (1 << COM1A1) | (1 << COM1A0); } while(0)
#define SOUND_OFF do { TCCR1A = 0; } while(0)

typedef struct
{
    uint16_t freq;
    uint8_t delay;
    uint8_t anz;
    int16_t inc;
} ton_t;

typedef struct sound_t
{
    uint8_t delay;
    uint8_t anz;
    ton_t ton;
} sound_t;

const ton_t TON[] PROGMEM = 
{
    { FREQ(700), 1, 25,  -300},
    { FREQ(650), 2, 25,  1200}
};

sound_t sdata;

void 
sound (uint8_t id)
{
    memcpy_P (&sdata.ton, &TON[id], sizeof (ton_t));
    sdata.delay = sdata.ton.delay;
    sdata.anz   = sdata.ton.anz;
}

void 
job_sound (sound_t * s)
{
    uint8_t delay = s->delay;
    
    if (!delay)
        return;
        
    if (--delay)
    {
        s->delay = delay;
        return;
    }
    
    // delay abgelaufen
    
    uint8_t anz = s->anz;
    
    if (!anz)
        return;
        
    if (!--anz)
    {
        // anz abgelaufen
        
        s->delay = 0;
        SOUND_OFF;
        s->anz   = 0;
        return;
    }
    
    s->delay = s->ton.delay;

    uint16_t icr = s->ton.freq;
    ICR1 = icr;   
    OCR1A = icr/2;
    OCR1B = icr/2;
    
    s->ton.freq += s->ton.inc;
    SOUND_ON;
    s->anz = anz;
}
