#include "compat/pgmspace.h"

#include "textmenu.h"
#include "menu.h"
#include "frame.h"
#include "text.h"
#include "testbild.h"

#include "rc5-codes.h"

const char LABELS_TEXTMENU[] PROGMEM = 
    "Uhr"       "\n"
    "Texte"     "\n"
    "Testbild"  "\n"
    "Sys-Config""\n"
    "Snake"     "\n"
    "Asteroids" "\n"
    "Schoner"   "\n"
    "DCF-Oszi"  "\n";
    
const char LABELS_CONFIGMENU[] PROGMEM = 
    "Bildschirmschoner""\n"
    "Pixelrate"        "\n"
    "Pixel pro Frame"  "\n"
    "IRQ-Rate";

const textmenu_t textmenus[] PROGMEM =
{
    [ITEMID_TEXTMENU - ITEMIDS] = 
    {
        .item_count = 8,
        .labels = LABELS_TEXTMENU,
        .items = 
        {
            ITEMID_UHR,
            ITEMID_TEXT,
            ITEMID_TEST,
            ITEMID_CONFIG,
            ITEMID_SNAKE,
            ITEMID_ASTEROIDS,
            ITEMID_SCHONER,
            ITEMID_DCFOSZI
        }
    }
    ,
    [ITEMID_CONFIG - ITEMIDS] = 
    {
        .item_count = 4,
        .labels = LABELS_CONFIGMENU,
        .items = 
        {
            ITEMID_CONFIG,
            ITEMID_CONFIG,
            ITEMID_CONFIG,
            ITEMID_CONFIG
        }
    }
};

static void 
onRedraw_textmenu (menu_t * m, text_frame_t * tb)
{
    uint8_t zoom = 2;
    
    uint8_t space_x = VF_SPACE_X(zoom);
    uint8_t space_y = VF_SPACE_Y(zoom);
    
    upixel8_t start = {50, 210 - space_y};
    
    // String #0 enthlt den Auswahl-Pfeil
    text_t * t = & tb->text[0];
    
    RELOAD ("y", t);
    
    t->start = start;
    t->text_in_flash = 0;
    t->proportional  = 0;
    t->zoom = zoom;
    
    uint8_t down = m->sel_id;
    
    char *str = tb->str;
    t->pstr  = str;
    
    for (uint8_t i=0; i < down; i++)
        *str++ = '\n'; // 1 Zeile tiefer
        
    *str++ = '>';  // Pfeil ->
    *str++ = '\0'; // Stringende
    
    // String #1 enthlt die Label-Texte
    t++;
    
    uint8_t dx = 3 *  space_x;
    start.x +=  dx/2;
    
    t->start = start;
    t->zoom  = zoom;
    t->pstr  = (char*) pgm_read_addr (& textmenus[m->menu_id].labels);
    t->text_in_flash = 1;
    t->proportional  = 1;
}

static void 
onEnter_textmenu (menu_t * m)
{
    m->sel_id = 0;
}

static uint8_t 
onKey_textmenu (menu_t * m, uint8_t code)
{
    int8_t id = m->sel_id;
    
    if (code == RC5_RIGHT) 
    {
        // Welches Item ist zu aktivieren?
        uint8_t i = id;
        i = pgm_read_byte (& textmenus[m->menu_id].items[i]);
        menu_activate_item (m, i);
        return 1;
    }
    
    if (code == RC5_DOWN2)   id++;
    if (code == RC5_UP2)     id--;
    
    if (id < 0)                id = 0;
    if ((uint8_t) id >= m->item_count)   
        id = m->item_count-1;
    
    m->sel_id = id;
    
    return 1;
}

const menuitem_t menuitem_textmenu PROGMEM = 
{
    .onPixel     = onPixel_text, // --> text.c
    .onEnter     = onEnter_textmenu,
    .onKey       = onKey_textmenu,
    .onRedraw    = onRedraw_textmenu,
    .bufsize     = sizeof (text_frame_t),
    .data        = & menu,
    .redrawOn    = 0
};
